## (c) 2016 Sven Wilksen
## ascii camera photo booth script for the raspberry pi
##
## This program is released under The MIT License (MIT)
#
# Copyright (c) 2016 Sven Wilksen
# Permission is hereby granted, free of charge, to any person 
# obtaining a copy of this software and associated documentation 
# files (the "Software"), to deal in the Software without 
# restriction, including without limitation the rights to use, 
# copy, modify, merge, publish, distribute, sublicense, and/or 
# sell copies of the Software, and to permit persons to whom 
# the Software is furnished to do so, subject to the following 
# conditions:
# The above copyright notice and this permission notice shall 
# be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
# OTHER DEALINGS IN THE SOFTWARE.

from picamera import PiCamera
from time import sleep
from subprocess import call
import RPi.GPIO as GPIO
import pygame

green_pressed = 0;
red_pressed = 0;

# callback routines
def green_button_callback(ch):
        global green_pressed;
        green_pressed = 1;
##        global img_waiting;
##        global imgcanvas;
##        # if an image was already taken, the green button
##        # confirms to print
##        
##        if (image_printing == True):
##                print('Cannot take another picture while printing');
##        elif (image_taken == True):
##                image_printing = True;
##                # convert to ascii and echo to printer
##                # display overlay saying "printing...?"
##                imgcanvas.blit(img_waiting,(0,0));
##                pygame.display.flip();
##                print('Printing image');
##                call(['convert','/dev/shm/cam_capture/image.jpg','-rotate','90','/dev/shm/cam_capture/img_rot90.jpg']);
##                print('Rotated');
##                call(['jp2a','/dev/shm/cam_capture/img_rot90.jpg','--background=light','--width=80','--output=/dev/shm/cam_capture/image.txt']);
##                print('Converted to ascii');
##                # restart preview
##                camera.start_preview(fullscreen=False, window=(100,20,640,580));
##                image_taken = False;
##                image_printing = False;
##        else:
##                # no image pending -> say cheese
##                print('Image capture');
##                camera.annotate_text = 'Lachen!';
##                sleep(1);
##                camera.annotate_text = ' ';
##                camera.stop_preview();
##                camera.capture('/dev/shm/cam_capture/image.jpg');
##                image_taken = True;
##                # display the image taken
##                last_image = pygame.image.load('/dev/shm/cam_capture/image.jpg');
##                imgcanvas.blit(last_image,(0,0));
##                pygame.display.flip();

def red_button_callback(ch):
        global red_pressed;
        red_pressed = 1;
##        global image_taken;
##        global image_printing;
##        # if an image was already taken, we close the
##        # preview and delete it
##        if (image_taken == True):
##                print('Image cleared');
##                image_taken = False;
##                # remove last taken image from buffer and replace with idle screen
##                imgcanvas.blit(img_waiting,(0,0));
##                pygame.display.flip();
##                # restart preview
##                camera.start_preview(fullscreen=False, window=(100,20,640,580));
                

## main programm

# filenames
jpg_path = '/dev/shm/image.jpg';
jpg_processed_path = '/dev/shm/processed_image.jpg';
txt_path = '/dev/shm/image.txt';
#device_path = '/dev/shm/print.txt';
device_path = '/dev/usb/lp0';

# GPIO configuration
GPIO.setmode(GPIO.BOARD);
GPIO.setup(40, GPIO.IN, pull_up_down=GPIO.PUD_UP);
GPIO.setup(38, GPIO.IN, pull_up_down=GPIO.PUD_UP);

#image display configuration
pygame.init();
imgcanvas = pygame.display.set_mode((0,0));
img_waiting = pygame.image.load('./msg_images/idle_screen.png');
img_printing = pygame.image.load('./msg_images/print_screen.png');
imgcanvas.blit(img_waiting,(0,0));
pygame.display.flip();


# camera configuration
camera = PiCamera();
camera.flash_mode = 'on';
camera.start_preview();

# callback configuration
GPIO.add_event_detect(40, GPIO.FALLING, callback=green_button_callback, bouncetime=500);
GPIO.add_event_detect(38, GPIO.FALLING, callback=red_button_callback, bouncetime=500);

# state machine
iState = 0; #idle
while 1:
	if (iState == 0): #idle
		if (green_pressed == 1):
			print('Entering capture state');
		        iState = 10;
		green_pressed = 0;
		red_pressed = 0;
        
	elif (iState == 10): #capture
		print('Image capture');
		camera.annotate_text = 'Lachen!';
		sleep(1);
		camera.annotate_text = ' ';
		camera.stop_preview();
		camera.capture(jpg_path);
		# display the image taken
		last_image = pygame.image.load(jpg_path);
		imgcanvas.blit(last_image,(0,0));
		pygame.display.flip();

		iState = 20; # wait for print confirm image idle step
		red_pressed = 0;
		green_pressed = 0;

	elif (iState == 20): # image idle
		if (green_pressed == 1): #gruener Knopf: druggn
			print('Drucken');
			#statusnachricht anzeigen: drucken
			imgcanvas.blit(img_printing,(0,0));
			pygame.display.flip();
			iState = 30; # 
		elif( red_pressed == 1): #roter Knopp: Abbrechen
			# standardbild laden
			imgcanvas.blit(img_waiting,(0,0));
			pygame.display.flip();
			# preview an
			camera.start_preview();
			print('Abbrechen');
			iState = 0;

		red_pressed = 0;
		green_pressed = 0;
    
	elif (iState == 30):        # print
		red_pressed = 0;
		green_pressed = 0;

		print('Printing image');
		call(['convert',jpg_path,'-rotate','90','-contrast-stretch','8%',jpg_processed_path]);
		print('Rotated');
		call(['jp2a',jpg_processed_path,'--background=light','--width=80','--output='+txt_path]);
		print('Converted to ascii');
		print('Start printing');
		#print_device=open(device_path,'w');
		#call(['cat',txt_path],stdout=print_device);
		#print_device.close();
		call(['./print2lp0.sh'], shell=True);
		sleep(60);
		#preview anreissen
		camera.start_preview();
		print('Return to idle');
		iState = 0; #fertig, wieder in Idle
    
	else: #ungueltiger Zustand
		print('Ungueltiger Zustand');
		red_pressed = 0;
		green_pressed = 0;
		iState = 0;


#camera.start_preview(fullscreen=False, window=(100,20,640,580));
#sleep(10);
#camera.stop_preview();
#camera.capture('image.jpg');
